#!/usr/bin/env python3
"""
Enhanced AI Agent System - Command Line Interface

This script provides a command-line interface for interacting with the AI agent.
"""

import sys
import os
import argparse
import time

# Add the parent directory to the path so we can import our modules
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

from core.agent import AIAgent
from utils.formatter import EnhancedOutputFormatter

def interactive_mode():
    """Enhanced interactive mode for testing the agent"""
    formatter = EnhancedOutputFormatter(enable_animations=True)
    
    # Clear screen and show banner
    formatter.clear_screen()
    formatter.print_banner("Interactive AI Agent")
    print()
    
    print(formatter.format_simple_message("Type 'quit' to exit, 'help' for commands", "info"))
    print(f"{formatter._get_color('cyan')}{'='*50}{formatter._get_color('end')}")
    print()
    
    # Initialize the agent
    formatter.start_progress("Initializing AI Agent...")
    agent = AIAgent(
        model_name="gpt-4o",
        memory_persist_path="./interactive_memory",
        enable_reasoning_display=True,
        realtime_output=True,
        minimal_output=False
    )
    formatter.stop_progress()
    
    print(formatter.format_simple_message("AI Agent ready for interaction!", "success"))
    print()
    
    while True:
        try:
            # Enhanced prompt
            prompt = f"{formatter._get_color('bold')}{formatter._get_color('green')}🔍{formatter._get_color('end')} Enter your query: "
            query = input(prompt).strip()
            
            if query.lower() in ['quit', 'exit', 'q']:
                print(formatter.format_simple_message("👋 Goodbye! Thanks for using the AI Agent!", "info"))
                break
            
            elif query.lower() == 'help':
                print_enhanced_help(formatter)
                continue
            
            elif query.lower() == 'stats':
                formatter.start_progress("Generating statistics...")
                stats = agent.get_memory_stats()
                formatter.stop_progress()
                print(formatter.format_memory_stats(stats))
                continue
            
            elif query.lower() == 'tools':
                formatter.start_progress("Loading tools...")
                tool_info = agent.tool_registry.get_tool_info()
                formatter.stop_progress()
                print(formatter.format_tool_list(tool_info))
                continue
            
            elif query.lower() == 'clear':
                formatter.start_progress("Clearing memory...")
                agent.clear_memory()
                formatter.stop_progress()
                print(formatter.format_simple_message("🧹 Memory cleared successfully!", "success"))
                continue
            
            elif query.lower() == 'banner':
                formatter.print_banner("Interactive AI Agent")
                continue
            
            elif not query:
                continue
            
            # Process the query with enhanced feedback
            print(f"\n{formatter._get_color('blue')}🔄{formatter._get_color('end')} Processing query...")
            formatter.start_progress("AI Agent is thinking...")
            
            response = agent.process_query(query)
            
            formatter.stop_progress()
            
            # Format and display the enhanced response
            formatted_response = formatter.format_agent_response(response)
            print(formatted_response)
            
        except KeyboardInterrupt:
            formatter.stop_progress()
            print(formatter.format_simple_message("\n⚠️  Operation cancelled by user", "warning"))
            break
        except Exception as e:
            formatter.stop_progress()
            print(formatter.format_simple_message(f"❌ Error: {str(e)}", "error"))


def print_enhanced_help(formatter):
    """Print enhanced help information"""
    help_text = f"""
{formatter._get_color('bold')}{formatter._get_color('purple')}🆘 Available Commands:{formatter._get_color('end')}
  
  {formatter._get_color('green')}•{formatter._get_color('end')} help     - Show this help message
  {formatter._get_color('green')}•{formatter._get_color('end')} stats    - Display memory statistics
  {formatter._get_color('green')}•{formatter._get_color('end')} tools    - List available tools
  {formatter._get_color('green')}•{formatter._get_color('end')} clear    - Clear agent memory
  {formatter._get_color('green')}•{formatter._get_color('end')} banner   - Show the banner
  {formatter._get_color('green')}•{formatter._get_color('end')} quit     - Exit interactive mode
  
{formatter._get_color('bold')}{formatter._get_color('cyan')}🔧 Example Queries:{formatter._get_color('end')}
  
  {formatter._get_color('yellow')}•{formatter._get_color('end')} "Calculate 15 * 23 using Python"
  {formatter._get_color('yellow')}•{formatter._get_color('end')} "List files in the current directory"
  {formatter._get_color('yellow')}•{formatter._get_color('end')} "Browse to https://example.com"
  {formatter._get_color('yellow')}•{formatter._get_color('end')} "Write a short story about robots"
  {formatter._get_color('yellow')}•{formatter._get_color('end')} "Compare file1.txt and file2.txt"
  {formatter._get_color('yellow')}•{formatter._get_color('end')} "Take a screenshot of google.com"
  {formatter._get_color('yellow')}•{formatter._get_color('end')} "Scrape product information from amazon.com"
    """
    print(help_text)


def main_cli():
    """Main CLI entry point"""
    parser = argparse.ArgumentParser(description="Enhanced AI Agent System")
    parser.add_argument("--interactive", "-i", action="store_true", 
                       help="Run in interactive mode")
    parser.add_argument("--no-animations", action="store_true",
                       help="Disable animations for better performance")
    
    args = parser.parse_args()
    
    if args.interactive:
        interactive_mode()
    else:
        # You might want to add a default behavior when no flags are provided
        print("Welcome to the AI Agent System. Use --interactive to start a session.")
        parser.print_help()

if __name__ == "__main__":
    main_cli()